// Network Racer Track Editor
// By Jum Hig '97
// started: 28/10/97
// updated: 19/11/97

// Notes: see text file Tred.txt for instructions

#include <stdlib.h>
#include <stdio.h>

#include "allegro.h"
#include "racer.h"
#include "racerdat.h"


typedef struct {
	int block;
   int rotn;
} tile_type;

tile_type tile[20][20];
Point2D_type checkpoint[20];
int numcheckpoints;
char mystring[80];
PALETTE mypalette;
RGB white = { 63, 63, 63 };
BITMAP *buffer;
BITMAP *temp_bmp;
DATAFILE *datafile;

// prototypes for functions 
int Save_Tile_Map(void);
int Load_Tile_Map(void);
void DrawCheckPoints(void);

void main()
{
	int i, j, k, x, y, n;
   int curblock, blk_rotn, gameover;
   int offset_x, offset_y;

   numcheckpoints = 0;

	// Initialise Allegro etc
	allegro_init();
	install_timer();
	install_keyboard();
   install_mouse();

	// load datafile
	printf("Welcome to Jum's Net Racer Track Editor\n");
	printf("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n\n");
	printf("See 'Tred.txt' for instructions.\n\n");
   printf("Tile type element size %d\n", sizeof(tile));
	printf("Loading and decompressing datafile...\n");
	datafile = load_datafile("racer.dat");
	if(!datafile) {
		allegro_exit();
		printf("Error loading datafile racerdat.dat!\n\n");
		exit(1);
	}


	delay(2000);
   // clear tilemap (avoids problems later)
   for(i=0; i < 20; i++) {
   	for(j=0; j < 20; j++) tile[i][j].block = 0;
	}

	// set graph mode and palette
	set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0);
	set_palette(datafile[PAL_RACER].dat);
   set_mouse_sprite(datafile[BMP_3DMOUSE].dat);

   buffer = create_bitmap(SCREEN_W, SCREEN_H);
   temp_bmp = create_bitmap(80, 80);
   gui_fg_color = 2; gui_bg_color = 3;

	// ---------------------------------------------------------------------
	// --------------------------- Main Prog Loop --------------------------
	// ---------------------------------------------------------------------

   curblock = 0; x = 0; y = 0;
   blk_rotn = 0;
   clear(buffer);
	// draw block select blocks
	for(i=0; i<=MAX_BLOCKS; i++) {
		stretch_blit(datafile[BLK00 + i].dat, buffer, 0, 0, 80, 80, 510 + (i % 3)*42, 104 + (i/3)*42, 40, 40);
	}

   // ****************************** main loop *****************************
   gameover = 0;

   while(!gameover) {

		// Get keyboard  User input and respond
		if(key[KEY_LEFT] && (x > 0)) x--;
		if(key[KEY_RIGHT] && (x < 15)) x++ ;
      if(key[KEY_UP] && (y > 0)) y--;
      if(key[KEY_DOWN] && (y < 15)) y++;
		if(key[KEY_A] && (curblock < MAX_BLOCKS)) curblock++;
		if(key[KEY_Z] && (curblock > 0)) curblock--;
      if(key[KEY_SPACE]) { 				// clear block and draw it
         clear(temp_bmp);
      	rotate_sprite(temp_bmp, datafile[BLK00 + curblock].dat, 0, 0, itofix(blk_rotn*64));
			stretch_blit(temp_bmp, buffer, 0, 0, 80, 80, x*24, y*24, 24, 24);
         tile[x][y].block = curblock;
         tile[x][y].rotn = blk_rotn*64;

      	DrawCheckPoints();
		}
		if(key[KEY_PLUS_PAD]) {
      	blk_rotn++;
         if(blk_rotn > 3) blk_rotn = 0;
      	rest(200);
		}
      if(key[KEY_MINUS_PAD]) {
      	blk_rotn--;
         if(blk_rotn < 0) blk_rotn = 3;
         rest(200);
		}
      if((key[KEY_INSERT]) && (numcheckpoints < 19) && (mouse_x < 480)) {
      	checkpoint[numcheckpoints].x = mouse_x;
         checkpoint[numcheckpoints].y = mouse_y;
         numcheckpoints++;
         DrawCheckPoints();
         circle(screen, mouse_x, mouse_y, 32, 3);
         rest(200);
		}
		if((key[KEY_DEL]) && (numcheckpoints > 0)) {
	      for(i=0; i<20; i++) {
   	   	for(j=0; j<20; j++) {
      	   	clear(temp_bmp);
      			rotate_sprite(temp_bmp, datafile[BLK00 + tile[i][j].block].dat, 0, 0, itofix(tile[i][j].rotn));
					stretch_blit(temp_bmp, buffer, 0, 0, 80, 80, i*24, j*24, 24, 24);
				}
			}
      	numcheckpoints--;
         DrawCheckPoints();
		}
		if(key[KEY_F9]) i = Load_Tile_Map();
      if(key[KEY_F10]) i = Save_Tile_Map();
		if(key[KEY_ESC]) {
      	gameover = 1;
			exit(0);
		}

      // get mouse input and respond
      if(mouse_b == 1) {
      	// is mouse in select block?
      	if((mouse_x > 509) && (mouse_y > 103)) {
         	curblock = 3 * ((mouse_y - 104) / 42) + ((mouse_x-510) / 42);

            if(curblock > MAX_BLOCKS) curblock = MAX_BLOCKS;
         }
         // is mouse in tile map area?
      	if((mouse_x < 480) && (mouse_y < 480)) {
				x = mouse_x / 24;
            y = mouse_y / 24;
         	clear(temp_bmp);
      		rotate_sprite(temp_bmp, datafile[BLK00 + curblock].dat, 0, 0, itofix(blk_rotn*64));
				stretch_blit(temp_bmp, buffer, 0, 0, 80, 80, x*24, y*24, 24, 24);
         	tile[x][y].block = curblock;
         	tile[x][y].rotn = blk_rotn*64;
         	DrawCheckPoints();
			}


      }
      if(mouse_b == 2) {
      	blk_rotn++;
         if(blk_rotn > 3) blk_rotn = 0;
      }
  	   sprintf(mystring, "%2d %2d %3d %3d", x, y, mouse_x, mouse_y);
      textout(buffer, font, mystring, 520, 0, 2);
      rectfill(buffer, 540, 20, 639, 99, 0);
		rect(buffer, 539, 19, 620, 100, 14);
      rotate_sprite(buffer, datafile[BLK00 + curblock].dat, 540, 20, itofix(blk_rotn*64));

      // blit buffer to screen
      show_mouse(NULL);
      blit(buffer, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
      // draw brush block & cursor rectangle direct to screen
		rect(screen, x*24, y*24, x*24+23, y*24+23, 3);
      //sprintf(mystring, "%d", curblock);
      //textout_centre(screen, font, mystring, x*30 + 10, y*30+10, 2);
      show_mouse(screen);
   	rest(100);
	}

	text_mode(-1);
	sprintf(mystring,"GOODBYE");
	textout_centre(screen, datafile[FNT_JUM].dat, mystring, 160, 80, -1);
	rest(5000);

	exit(0);
}
/* -------------------------- End of Main ------------------------------*/

int Save_Tile_Map(void) {
	// save tile map
	int i, j, nsc;
	char cmybuffer[800];
	FILE *pftilemap;

	sprintf(mystring, "*.map");
   i = file_select("Save Map", mystring, "MAP");

	pftilemap = fopen(mystring, "w");

	if (setvbuf(pftilemap, cmybuffer, _IOFBF, MYBUFSIZE) == 0) {
		//printf("pftilmap buffer created.\n");
      // save checkpoints
      nsc = fprintf(pftilemap, "%2d\n", numcheckpoints);
      for(i=0; i<numcheckpoints; i++) {
      	nsc = fprintf(pftilemap, "%4d %4d\n", checkpoint[i].x, checkpoint[i].y);
		}
      // save tilemap
      for(i=0; i<20; i++) {
      	for(j=0; j<20; j++) {
				nsc = fprintf(pftilemap, "%2d %3d\n", tile[i][j].block, tile[i][j].rotn);
			}
		}
		//printf("Tilemap saved successfully.\n");
	}
	else
		fprintf(stderr, "pftilemap buffer allocation error!\n");
	
	fclose(pftilemap);

}

int Load_Tile_Map(void) {
	// load tile map
	int i, j, nsc;
	char cmybuffer[800];
	FILE *pftilemap;

	sprintf(mystring, "*.map");
   i = file_select("Load Map", mystring, "MAP");

	pftilemap = fopen(mystring, "r");

	if (setvbuf(pftilemap, cmybuffer, _IOFBF, MYBUFSIZE) == 0) {
		//printf("pftilmap buffer created.\n");
      // load checkpoints
      nsc = fscanf(pftilemap, "%2d", &numcheckpoints);
      for(i=0; i<numcheckpoints; i++) {
      	nsc = fscanf(pftilemap, "%4d %4d", &checkpoint[i].x, &checkpoint[i].y);
		}
      // load tilemap
      for(i=0; i<20; i++) {
      	for(j=0; j<20; j++) {
				nsc = fscanf(pftilemap, "%2d %3d", &tile[i][j].block, &tile[i][j].rotn);
			}
		}
		//printf("Tilemap loaded successfully.\n");

      // draw tilemap
      clear(buffer);
      vsync();
      for(i=0; i<20; i++) {
      	for(j=0; j<20; j++) {
         	clear(temp_bmp);
      		rotate_sprite(temp_bmp, datafile[BLK00 + tile[i][j].block].dat, 0, 0, itofix(tile[i][j].rotn));
				stretch_blit(temp_bmp, buffer, 0, 0, 80, 80, i*24, j*24, 24, 24);
			}
		}

      // draw checkpoint flags
      DrawCheckPoints();

		// draw block select blocks
		for(i=0; i<=MAX_BLOCKS; i++) {
			stretch_blit(datafile[BLK00 + i].dat, buffer, 0, 0, 80, 80, 510 + (i % 3)*42, 104 + (i/3)*42, 40, 40);
		}

	}
	else
		printf("pftilemap buffer allocation error!\n");
	
	fclose(pftilemap);
}


// Refresh draw all checkpoint flags
void DrawCheckPoints(void) {
	int i, j;

	// draw checkpoint flags
	for(i=0; i<numcheckpoints; i++) {
		draw_rle_sprite(buffer, datafile[RLE_FLAG].dat, checkpoint[i].x - 16, checkpoint[i].y - 16);
	}

	sprintf(mystring,"CP: %2d", numcheckpoints);
	textout(buffer, font, mystring, 510, 472, -1);

}

